# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.beta.clusters import storage_create_params, storage_update_params
from ....types.beta.clusters.cluster_storage import ClusterStorage
from ....types.beta.clusters.storage_list_response import StorageListResponse
from ....types.beta.clusters.storage_delete_response import StorageDeleteResponse

__all__ = ["StorageResource", "AsyncStorageResource"]


class StorageResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> StorageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return StorageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> StorageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return StorageResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        region: str,
        size_tib: int,
        volume_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClusterStorage:
        """
        Instant Clusters supports long-lived, resizable in-DC shared storage with user
        data persistence. You can dynamically create and attach volumes to your cluster
        at cluster creation time, and resize as your data grows. All shared storage is
        backed by multi-NIC bare metal paths, ensuring high-throughput and low-latency
        performance for shared storage.

        Args:
          region: Region name. Usable regions can be found from `client.clusters.list_regions()`

          size_tib: Volume size in whole tebibytes (TiB).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/clusters/storages",
            body=maybe_transform(
                {
                    "region": region,
                    "size_tib": size_tib,
                    "volume_name": volume_name,
                },
                storage_create_params.StorageCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClusterStorage,
        )

    def retrieve(
        self,
        volume_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClusterStorage:
        """
        Retrieve information about a specific shared volume.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not volume_id:
            raise ValueError(f"Expected a non-empty value for `volume_id` but received {volume_id!r}")
        return self._get(
            f"/clusters/storages/{volume_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClusterStorage,
        )

    def update(
        self,
        *,
        size_tib: int | Omit = omit,
        volume_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClusterStorage:
        """
        Update the configuration of an existing shared volume.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._put(
            "/clusters/storages",
            body=maybe_transform(
                {
                    "size_tib": size_tib,
                    "volume_id": volume_id,
                },
                storage_update_params.StorageUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClusterStorage,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StorageListResponse:
        """List all shared volumes."""
        return self._get(
            "/clusters/storages",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StorageListResponse,
        )

    def delete(
        self,
        volume_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StorageDeleteResponse:
        """Delete a shared volume.

        Note that if this volume is attached to a cluster,
        deleting will fail.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not volume_id:
            raise ValueError(f"Expected a non-empty value for `volume_id` but received {volume_id!r}")
        return self._delete(
            f"/clusters/storages/{volume_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StorageDeleteResponse,
        )


class AsyncStorageResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncStorageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return AsyncStorageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncStorageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return AsyncStorageResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        region: str,
        size_tib: int,
        volume_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClusterStorage:
        """
        Instant Clusters supports long-lived, resizable in-DC shared storage with user
        data persistence. You can dynamically create and attach volumes to your cluster
        at cluster creation time, and resize as your data grows. All shared storage is
        backed by multi-NIC bare metal paths, ensuring high-throughput and low-latency
        performance for shared storage.

        Args:
          region: Region name. Usable regions can be found from `client.clusters.list_regions()`

          size_tib: Volume size in whole tebibytes (TiB).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/clusters/storages",
            body=await async_maybe_transform(
                {
                    "region": region,
                    "size_tib": size_tib,
                    "volume_name": volume_name,
                },
                storage_create_params.StorageCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClusterStorage,
        )

    async def retrieve(
        self,
        volume_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClusterStorage:
        """
        Retrieve information about a specific shared volume.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not volume_id:
            raise ValueError(f"Expected a non-empty value for `volume_id` but received {volume_id!r}")
        return await self._get(
            f"/clusters/storages/{volume_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClusterStorage,
        )

    async def update(
        self,
        *,
        size_tib: int | Omit = omit,
        volume_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClusterStorage:
        """
        Update the configuration of an existing shared volume.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._put(
            "/clusters/storages",
            body=await async_maybe_transform(
                {
                    "size_tib": size_tib,
                    "volume_id": volume_id,
                },
                storage_update_params.StorageUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClusterStorage,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StorageListResponse:
        """List all shared volumes."""
        return await self._get(
            "/clusters/storages",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StorageListResponse,
        )

    async def delete(
        self,
        volume_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StorageDeleteResponse:
        """Delete a shared volume.

        Note that if this volume is attached to a cluster,
        deleting will fail.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not volume_id:
            raise ValueError(f"Expected a non-empty value for `volume_id` but received {volume_id!r}")
        return await self._delete(
            f"/clusters/storages/{volume_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StorageDeleteResponse,
        )


class StorageResourceWithRawResponse:
    def __init__(self, storage: StorageResource) -> None:
        self._storage = storage

        self.create = to_raw_response_wrapper(
            storage.create,
        )
        self.retrieve = to_raw_response_wrapper(
            storage.retrieve,
        )
        self.update = to_raw_response_wrapper(
            storage.update,
        )
        self.list = to_raw_response_wrapper(
            storage.list,
        )
        self.delete = to_raw_response_wrapper(
            storage.delete,
        )


class AsyncStorageResourceWithRawResponse:
    def __init__(self, storage: AsyncStorageResource) -> None:
        self._storage = storage

        self.create = async_to_raw_response_wrapper(
            storage.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            storage.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            storage.update,
        )
        self.list = async_to_raw_response_wrapper(
            storage.list,
        )
        self.delete = async_to_raw_response_wrapper(
            storage.delete,
        )


class StorageResourceWithStreamingResponse:
    def __init__(self, storage: StorageResource) -> None:
        self._storage = storage

        self.create = to_streamed_response_wrapper(
            storage.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            storage.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            storage.update,
        )
        self.list = to_streamed_response_wrapper(
            storage.list,
        )
        self.delete = to_streamed_response_wrapper(
            storage.delete,
        )


class AsyncStorageResourceWithStreamingResponse:
    def __init__(self, storage: AsyncStorageResource) -> None:
        self._storage = storage

        self.create = async_to_streamed_response_wrapper(
            storage.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            storage.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            storage.update,
        )
        self.list = async_to_streamed_response_wrapper(
            storage.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            storage.delete,
        )



from typing import Any, Dict, List, Union, Literal

import click
from tabulate import tabulate

from together import Together, omit
from together.lib.cli.api._utils import handle_api_errors


@click.command()
@click.option(
    "--status",
    type=click.Choice(["pending", "queued", "running", "completed", "error", "user_error"]),
    help="Filter by job status.",
)
@click.option(
    "--limit",
    type=int,
    help="Limit number of results (max 100).",
)
@click.pass_context
@handle_api_errors("Evals")
def list(
    ctx: click.Context,
    status: Union[Literal["pending", "queued", "running", "completed", "error", "user_error"], None],
    limit: Union[int, None],
) -> None:
    """List evals"""

    client: Together = ctx.obj

    response = client.evals.list(status=status or omit, limit=limit or omit)

    display_list: List[Dict[str, Any]] = []
    for job in response:
        if job.parameters:
            model = job.parameters.get("model_to_evaluate", "")
            model_a = job.parameters.get("model_a", "")
            model_b = job.parameters.get("model_b", "")
        else:
            model = ""
            model_a = ""
            model_b = ""

        display_list.append(
            {
                "Workflow ID": job.workflow_id or "",
                "Type": job.type,
                "Status": job.status,
                "Created At": job.created_at or 0,
                "Model": model,
                "Model A": model_a,
                "Model B": model_b,
            }
        )

    table = tabulate(display_list, headers="keys", tablefmt="grid", showindex=True)
    click.echo(table)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..types import endpoint_list_params, endpoint_create_params, endpoint_update_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.autoscaling_param import AutoscalingParam
from ..types.dedicated_endpoint import DedicatedEndpoint
from ..types.endpoint_list_response import EndpointListResponse
from ..types.endpoint_list_avzones_response import EndpointListAvzonesResponse

__all__ = ["EndpointsResource", "AsyncEndpointsResource"]


class EndpointsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EndpointsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return EndpointsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EndpointsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return EndpointsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        autoscaling: AutoscalingParam,
        hardware: str,
        model: str,
        availability_zone: str | Omit = omit,
        disable_prompt_cache: bool | Omit = omit,
        disable_speculative_decoding: bool | Omit = omit,
        display_name: str | Omit = omit,
        inactive_timeout: Optional[int] | Omit = omit,
        state: Literal["STARTED", "STOPPED"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DedicatedEndpoint:
        """Creates a new dedicated endpoint for serving models.

        The endpoint will
        automatically start after creation. You can deploy any supported model on
        hardware configurations that meet the model's requirements.

        Args:
          autoscaling: Configuration for automatic scaling of the endpoint

          hardware: The hardware configuration to use for this endpoint

          model: The model to deploy on this endpoint

          availability_zone: Create the endpoint in a specified availability zone (e.g., us-central-4b)

          disable_prompt_cache: This parameter is deprecated and no longer has any effect.

          disable_speculative_decoding: Whether to disable speculative decoding for this endpoint

          display_name: A human-readable name for the endpoint

          inactive_timeout: The number of minutes of inactivity after which the endpoint will be
              automatically stopped. Set to null, omit or set to 0 to disable automatic
              timeout.

          state: The desired state of the endpoint

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/endpoints",
            body=maybe_transform(
                {
                    "autoscaling": autoscaling,
                    "hardware": hardware,
                    "model": model,
                    "availability_zone": availability_zone,
                    "disable_prompt_cache": disable_prompt_cache,
                    "disable_speculative_decoding": disable_speculative_decoding,
                    "display_name": display_name,
                    "inactive_timeout": inactive_timeout,
                    "state": state,
                },
                endpoint_create_params.EndpointCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DedicatedEndpoint,
        )

    def retrieve(
        self,
        endpoint_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DedicatedEndpoint:
        """
        Retrieves details about a specific endpoint, including its current state,
        configuration, and scaling settings.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not endpoint_id:
            raise ValueError(f"Expected a non-empty value for `endpoint_id` but received {endpoint_id!r}")
        return self._get(
            f"/endpoints/{endpoint_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DedicatedEndpoint,
        )

    def update(
        self,
        endpoint_id: str,
        *,
        autoscaling: AutoscalingParam | Omit = omit,
        display_name: str | Omit = omit,
        inactive_timeout: Optional[int] | Omit = omit,
        state: Literal["STARTED", "STOPPED"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DedicatedEndpoint:
        """Updates an existing endpoint's configuration.

        You can modify the display name,
        autoscaling settings, or change the endpoint's state (start/stop).

        Args:
          autoscaling: New autoscaling configuration for the endpoint

          display_name: A human-readable name for the endpoint

          inactive_timeout: The number of minutes of inactivity after which the endpoint will be
              automatically stopped. Set to 0 to disable automatic timeout.

          state: The desired state of the endpoint

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not endpoint_id:
            raise ValueError(f"Expected a non-empty value for `endpoint_id` but received {endpoint_id!r}")
        return self._patch(
            f"/endpoints/{endpoint_id}",
            body=maybe_transform(
                {
                    "autoscaling": autoscaling,
                    "display_name": display_name,
                    "inactive_timeout": inactive_timeout,
                    "state": state,
                },
                endpoint_update_params.EndpointUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DedicatedEndpoint,
        )

    def list(
        self,
        *,
        mine: bool | Omit = omit,
        type: Literal["dedicated", "serverless"] | Omit = omit,
        usage_type: Literal["on-demand", "reserved"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EndpointListResponse:
        """Returns a list of all endpoints associated with your account.

        You can filter the
        results by type (dedicated or serverless).

        Args:
          mine: If true, return only endpoints owned by the caller

          type: Filter endpoints by type

          usage_type: Filter endpoints by usage type

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/endpoints",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "mine": mine,
                        "type": type,
                        "usage_type": usage_type,
                    },
                    endpoint_list_params.EndpointListParams,
                ),
            ),
            cast_to=EndpointListResponse,
        )

    def delete(
        self,
        endpoint_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Permanently deletes an endpoint.

        This action cannot be undone.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not endpoint_id:
            raise ValueError(f"Expected a non-empty value for `endpoint_id` but received {endpoint_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/endpoints/{endpoint_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list_avzones(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EndpointListAvzonesResponse:
        """List all available availability zones."""
        return self._get(
            "/clusters/availability-zones",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EndpointListAvzonesResponse,
        )


class AsyncEndpointsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEndpointsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return AsyncEndpointsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEndpointsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return AsyncEndpointsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        autoscaling: AutoscalingParam,
        hardware: str,
        model: str,
        availability_zone: str | Omit = omit,
        disable_prompt_cache: bool | Omit = omit,
        disable_speculative_decoding: bool | Omit = omit,
        display_name: str | Omit = omit,
        inactive_timeout: Optional[int] | Omit = omit,
        state: Literal["STARTED", "STOPPED"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DedicatedEndpoint:
        """Creates a new dedicated endpoint for serving models.

        The endpoint will
        automatically start after creation. You can deploy any supported model on
        hardware configurations that meet the model's requirements.

        Args:
          autoscaling: Configuration for automatic scaling of the endpoint

          hardware: The hardware configuration to use for this endpoint

          model: The model to deploy on this endpoint

          availability_zone: Create the endpoint in a specified availability zone (e.g., us-central-4b)

          disable_prompt_cache: This parameter is deprecated and no longer has any effect.

          disable_speculative_decoding: Whether to disable speculative decoding for this endpoint

          display_name: A human-readable name for the endpoint

          inactive_timeout: The number of minutes of inactivity after which the endpoint will be
              automatically stopped. Set to null, omit or set to 0 to disable automatic
              timeout.

          state: The desired state of the endpoint

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/endpoints",
            body=await async_maybe_transform(
                {
                    "autoscaling": autoscaling,
                    "hardware": hardware,
                    "model": model,
                    "availability_zone": availability_zone,
                    "disable_prompt_cache": disable_prompt_cache,
                    "disable_speculative_decoding": disable_speculative_decoding,
                    "display_name": display_name,
                    "inactive_timeout": inactive_timeout,
                    "state": state,
                },
                endpoint_create_params.EndpointCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DedicatedEndpoint,
        )

    async def retrieve(
        self,
        endpoint_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DedicatedEndpoint:
        """
        Retrieves details about a specific endpoint, including its current state,
        configuration, and scaling settings.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not endpoint_id:
            raise ValueError(f"Expected a non-empty value for `endpoint_id` but received {endpoint_id!r}")
        return await self._get(
            f"/endpoints/{endpoint_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DedicatedEndpoint,
        )

    async def update(
        self,
        endpoint_id: str,
        *,
        autoscaling: AutoscalingParam | Omit = omit,
        display_name: str | Omit = omit,
        inactive_timeout: Optional[int] | Omit = omit,
        state: Literal["STARTED", "STOPPED"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DedicatedEndpoint:
        """Updates an existing endpoint's configuration.

        You can modify the display name,
        autoscaling settings, or change the endpoint's state (start/stop).

        Args:
          autoscaling: New autoscaling configuration for the endpoint

          display_name: A human-readable name for the endpoint

          inactive_timeout: The number of minutes of inactivity after which the endpoint will be
              automatically stopped. Set to 0 to disable automatic timeout.

          state: The desired state of the endpoint

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not endpoint_id:
            raise ValueError(f"Expected a non-empty value for `endpoint_id` but received {endpoint_id!r}")
        return await self._patch(
            f"/endpoints/{endpoint_id}",
            body=await async_maybe_transform(
                {
                    "autoscaling": autoscaling,
                    "display_name": display_name,
                    "inactive_timeout": inactive_timeout,
                    "state": state,
                },
                endpoint_update_params.EndpointUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DedicatedEndpoint,
        )

    async def list(
        self,
        *,
        mine: bool | Omit = omit,
        type: Literal["dedicated", "serverless"] | Omit = omit,
        usage_type: Literal["on-demand", "reserved"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EndpointListResponse:
        """Returns a list of all endpoints associated with your account.

        You can filter the
        results by type (dedicated or serverless).

        Args:
          mine: If true, return only endpoints owned by the caller

          type: Filter endpoints by type

          usage_type: Filter endpoints by usage type

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/endpoints",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "mine": mine,
                        "type": type,
                        "usage_type": usage_type,
                    },
                    endpoint_list_params.EndpointListParams,
                ),
            ),
            cast_to=EndpointListResponse,
        )

    async def delete(
        self,
        endpoint_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Permanently deletes an endpoint.

        This action cannot be undone.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not endpoint_id:
            raise ValueError(f"Expected a non-empty value for `endpoint_id` but received {endpoint_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/endpoints/{endpoint_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def list_avzones(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EndpointListAvzonesResponse:
        """List all available availability zones."""
        return await self._get(
            "/clusters/availability-zones",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EndpointListAvzonesResponse,
        )


class EndpointsResourceWithRawResponse:
    def __init__(self, endpoints: EndpointsResource) -> None:
        self._endpoints = endpoints

        self.create = to_raw_response_wrapper(
            endpoints.create,
        )
        self.retrieve = to_raw_response_wrapper(
            endpoints.retrieve,
        )
        self.update = to_raw_response_wrapper(
            endpoints.update,
        )
        self.list = to_raw_response_wrapper(
            endpoints.list,
        )
        self.delete = to_raw_response_wrapper(
            endpoints.delete,
        )
        self.list_avzones = to_raw_response_wrapper(
            endpoints.list_avzones,
        )


class AsyncEndpointsResourceWithRawResponse:
    def __init__(self, endpoints: AsyncEndpointsResource) -> None:
        self._endpoints = endpoints

        self.create = async_to_raw_response_wrapper(
            endpoints.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            endpoints.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            endpoints.update,
        )
        self.list = async_to_raw_response_wrapper(
            endpoints.list,
        )
        self.delete = async_to_raw_response_wrapper(
            endpoints.delete,
        )
        self.list_avzones = async_to_raw_response_wrapper(
            endpoints.list_avzones,
        )


class EndpointsResourceWithStreamingResponse:
    def __init__(self, endpoints: EndpointsResource) -> None:
        self._endpoints = endpoints

        self.create = to_streamed_response_wrapper(
            endpoints.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            endpoints.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            endpoints.update,
        )
        self.list = to_streamed_response_wrapper(
            endpoints.list,
        )
        self.delete = to_streamed_response_wrapper(
            endpoints.delete,
        )
        self.list_avzones = to_streamed_response_wrapper(
            endpoints.list_avzones,
        )


class AsyncEndpointsResourceWithStreamingResponse:
    def __init__(self, endpoints: AsyncEndpointsResource) -> None:
        self._endpoints = endpoints

        self.create = async_to_streamed_response_wrapper(
            endpoints.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            endpoints.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            endpoints.update,
        )
        self.list = async_to_streamed_response_wrapper(
            endpoints.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            endpoints.delete,
        )
        self.list_avzones = async_to_streamed_response_wrapper(
            endpoints.list_avzones,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .cluster import Cluster
from ..._models import BaseModel

__all__ = ["ClusterListResponse"]


class ClusterListResponse(BaseModel):
    clusters: List[Cluster]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from .storage import (
    StorageResource,
    AsyncStorageResource,
    StorageResourceWithRawResponse,
    AsyncStorageResourceWithRawResponse,
    StorageResourceWithStreamingResponse,
    AsyncStorageResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....types.beta import cluster_create_params, cluster_update_params
from ...._base_client import make_request_options
from ....types.beta.cluster import Cluster
from ....types.beta.cluster_list_response import ClusterListResponse
from ....types.beta.cluster_delete_response import ClusterDeleteResponse
from ....types.beta.cluster_list_regions_response import ClusterListRegionsResponse

__all__ = ["ClustersResource", "AsyncClustersResource"]


class ClustersResource(SyncAPIResource):
    @cached_property
    def storage(self) -> StorageResource:
        return StorageResource(self._client)

    @cached_property
    def with_raw_response(self) -> ClustersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return ClustersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ClustersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return ClustersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        billing_type: Literal["RESERVED", "ON_DEMAND"],
        cluster_name: str,
        driver_version: Literal["CUDA_12_5_555", "CUDA_12_6_560", "CUDA_12_6_565", "CUDA_12_8_570"],
        gpu_type: Literal["H100_SXM", "H200_SXM", "RTX_6000_PCI", "L40_PCIE", "B200_SXM", "H100_SXM_INF"],
        num_gpus: int,
        region: Literal["us-central-8", "us-central-4"],
        cluster_type: Literal["KUBERNETES", "SLURM"] | Omit = omit,
        duration_days: int | Omit = omit,
        shared_volume: cluster_create_params.SharedVolume | Omit = omit,
        volume_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Cluster:
        """Create an Instant Cluster on Together's high-performance GPU clusters.

        With
        features like on-demand scaling, long-lived resizable high-bandwidth shared
        DC-local storage, Kubernetes and Slurm cluster flavors, a REST API, and
        Terraform support, you can run workloads flexibly without complex infrastructure
        management.

        Args:
          cluster_name: Name of the GPU cluster.

          driver_version: NVIDIA driver version to use in the cluster.

          gpu_type: Type of GPU to use in the cluster

          num_gpus: Number of GPUs to allocate in the cluster. This must be multiple of 8. For
              example, 8, 16 or 24

          region: Region to create the GPU cluster in. Valid values are us-central-8 and
              us-central-4.

          duration_days: Duration in days to keep the cluster running.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/clusters",
            body=maybe_transform(
                {
                    "billing_type": billing_type,
                    "cluster_name": cluster_name,
                    "driver_version": driver_version,
                    "gpu_type": gpu_type,
                    "num_gpus": num_gpus,
                    "region": region,
                    "cluster_type": cluster_type,
                    "duration_days": duration_days,
                    "shared_volume": shared_volume,
                    "volume_id": volume_id,
                },
                cluster_create_params.ClusterCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Cluster,
        )

    def retrieve(
        self,
        cluster_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Cluster:
        """
        Retrieve information about a specific GPU cluster.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not cluster_id:
            raise ValueError(f"Expected a non-empty value for `cluster_id` but received {cluster_id!r}")
        return self._get(
            f"/clusters/{cluster_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Cluster,
        )

    def update(
        self,
        cluster_id: str,
        *,
        cluster_type: Literal["KUBERNETES", "SLURM"] | Omit = omit,
        num_gpus: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Cluster:
        """
        Update the configuration of an existing GPU cluster.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not cluster_id:
            raise ValueError(f"Expected a non-empty value for `cluster_id` but received {cluster_id!r}")
        return self._put(
            f"/clusters/{cluster_id}",
            body=maybe_transform(
                {
                    "cluster_type": cluster_type,
                    "num_gpus": num_gpus,
                },
                cluster_update_params.ClusterUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Cluster,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClusterListResponse:
        """List all GPU clusters."""
        return self._get(
            "/clusters",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClusterListResponse,
        )

    def delete(
        self,
        cluster_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClusterDeleteResponse:
        """
        Delete a GPU cluster by cluster ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not cluster_id:
            raise ValueError(f"Expected a non-empty value for `cluster_id` but received {cluster_id!r}")
        return self._delete(
            f"/clusters/{cluster_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClusterDeleteResponse,
        )

    def list_regions(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClusterListRegionsResponse:
        """List regions and corresponding supported driver versions"""
        return self._get(
            "/clusters/regions",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClusterListRegionsResponse,
        )


class AsyncClustersResource(AsyncAPIResource):
    @cached_property
    def storage(self) -> AsyncStorageResource:
        return AsyncStorageResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncClustersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return AsyncClustersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncClustersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return AsyncClustersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        billing_type: Literal["RESERVED", "ON_DEMAND"],
        cluster_name: str,
        driver_version: Literal["CUDA_12_5_555", "CUDA_12_6_560", "CUDA_12_6_565", "CUDA_12_8_570"],
        gpu_type: Literal["H100_SXM", "H200_SXM", "RTX_6000_PCI", "L40_PCIE", "B200_SXM", "H100_SXM_INF"],
        num_gpus: int,
        region: Literal["us-central-8", "us-central-4"],
        cluster_type: Literal["KUBERNETES", "SLURM"] | Omit = omit,
        duration_days: int | Omit = omit,
        shared_volume: cluster_create_params.SharedVolume | Omit = omit,
        volume_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Cluster:
        """Create an Instant Cluster on Together's high-performance GPU clusters.

        With
        features like on-demand scaling, long-lived resizable high-bandwidth shared
        DC-local storage, Kubernetes and Slurm cluster flavors, a REST API, and
        Terraform support, you can run workloads flexibly without complex infrastructure
        management.

        Args:
          cluster_name: Name of the GPU cluster.

          driver_version: NVIDIA driver version to use in the cluster.

          gpu_type: Type of GPU to use in the cluster

          num_gpus: Number of GPUs to allocate in the cluster. This must be multiple of 8. For
              example, 8, 16 or 24

          region: Region to create the GPU cluster in. Valid values are us-central-8 and
              us-central-4.

          duration_days: Duration in days to keep the cluster running.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/clusters",
            body=await async_maybe_transform(
                {
                    "billing_type": billing_type,
                    "cluster_name": cluster_name,
                    "driver_version": driver_version,
                    "gpu_type": gpu_type,
                    "num_gpus": num_gpus,
                    "region": region,
                    "cluster_type": cluster_type,
                    "duration_days": duration_days,
                    "shared_volume": shared_volume,
                    "volume_id": volume_id,
                },
                cluster_create_params.ClusterCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Cluster,
        )

    async def retrieve(
        self,
        cluster_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Cluster:
        """
        Retrieve information about a specific GPU cluster.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not cluster_id:
            raise ValueError(f"Expected a non-empty value for `cluster_id` but received {cluster_id!r}")
        return await self._get(
            f"/clusters/{cluster_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Cluster,
        )

    async def update(
        self,
        cluster_id: str,
        *,
        cluster_type: Literal["KUBERNETES", "SLURM"] | Omit = omit,
        num_gpus: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Cluster:
        """
        Update the configuration of an existing GPU cluster.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not cluster_id:
            raise ValueError(f"Expected a non-empty value for `cluster_id` but received {cluster_id!r}")
        return await self._put(
            f"/clusters/{cluster_id}",
            body=await async_maybe_transform(
                {
                    "cluster_type": cluster_type,
                    "num_gpus": num_gpus,
                },
                cluster_update_params.ClusterUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Cluster,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClusterListResponse:
        """List all GPU clusters."""
        return await self._get(
            "/clusters",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClusterListResponse,
        )

    async def delete(
        self,
        cluster_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClusterDeleteResponse:
        """
        Delete a GPU cluster by cluster ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not cluster_id:
            raise ValueError(f"Expected a non-empty value for `cluster_id` but received {cluster_id!r}")
        return await self._delete(
            f"/clusters/{cluster_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClusterDeleteResponse,
        )

    async def list_regions(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClusterListRegionsResponse:
        """List regions and corresponding supported driver versions"""
        return await self._get(
            "/clusters/regions",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClusterListRegionsResponse,
        )


class ClustersResourceWithRawResponse:
    def __init__(self, clusters: ClustersResource) -> None:
        self._clusters = clusters

        self.create = to_raw_response_wrapper(
            clusters.create,
        )
        self.retrieve = to_raw_response_wrapper(
            clusters.retrieve,
        )
        self.update = to_raw_response_wrapper(
            clusters.update,
        )
        self.list = to_raw_response_wrapper(
            clusters.list,
        )
        self.delete = to_raw_response_wrapper(
            clusters.delete,
        )
        self.list_regions = to_raw_response_wrapper(
            clusters.list_regions,
        )

    @cached_property
    def storage(self) -> StorageResourceWithRawResponse:
        return StorageResourceWithRawResponse(self._clusters.storage)


class AsyncClustersResourceWithRawResponse:
    def __init__(self, clusters: AsyncClustersResource) -> None:
        self._clusters = clusters

        self.create = async_to_raw_response_wrapper(
            clusters.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            clusters.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            clusters.update,
        )
        self.list = async_to_raw_response_wrapper(
            clusters.list,
        )
        self.delete = async_to_raw_response_wrapper(
            clusters.delete,
        )
        self.list_regions = async_to_raw_response_wrapper(
            clusters.list_regions,
        )

    @cached_property
    def storage(self) -> AsyncStorageResourceWithRawResponse:
        return AsyncStorageResourceWithRawResponse(self._clusters.storage)


class ClustersResourceWithStreamingResponse:
    def __init__(self, clusters: ClustersResource) -> None:
        self._clusters = clusters

        self.create = to_streamed_response_wrapper(
            clusters.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            clusters.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            clusters.update,
        )
        self.list = to_streamed_response_wrapper(
            clusters.list,
        )
        self.delete = to_streamed_response_wrapper(
            clusters.delete,
        )
        self.list_regions = to_streamed_response_wrapper(
            clusters.list_regions,
        )

    @cached_property
    def storage(self) -> StorageResourceWithStreamingResponse:
        return StorageResourceWithStreamingResponse(self._clusters.storage)


class AsyncClustersResourceWithStreamingResponse:
    def __init__(self, clusters: AsyncClustersResource) -> None:
        self._clusters = clusters

        self.create = async_to_streamed_response_wrapper(
            clusters.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            clusters.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            clusters.update,
        )
        self.list = async_to_streamed_response_wrapper(
            clusters.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            clusters.delete,
        )
        self.list_regions = async_to_streamed_response_wrapper(
            clusters.list_regions,
        )

    @cached_property
    def storage(self) -> AsyncStorageResourceWithStreamingResponse:
        return AsyncStorageResourceWithStreamingResponse(self._clusters.storage)



from together import Together

client = Together()

print("Listing all files")

# Print all files
files = client.files.list()
for file in files.data:
    print(file)

# Retrieve a file
if files.data and files.data[0]:
    print("Retrieving a file")

    file_id = files.data[0].id

    fileData = client.files.retrieve(file_id)

    print(fileData)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["EmbeddingCreateParams"]


class EmbeddingCreateParams(TypedDict, total=False):
    input: Required[Union[str, SequenceNotStr[str]]]
    """A string providing the text for the model to embed."""

    model: Required[
        Union[
            Literal[
                "WhereIsAI/UAE-Large-V1",
                "BAAI/bge-large-en-v1.5",
                "BAAI/bge-base-en-v1.5",
                "togethercomputer/m2-bert-80M-8k-retrieval",
            ],
            str,
        ]
    ]
    """The name of the embedding model to use.

    [See all of Together AI's embedding models](https://docs.together.ai/docs/serverless-models#embedding-models)
    """



import json
from pathlib import Path

from httpx import (
    Response,
)
from pytest_mock import MockerFixture

from together import Together
from together.types import (
    FileResponse,
)


def test_file_upload(mocker: MockerFixture, tmp_path: Path):
    # Mock the API requestor

    content = [{"text": "Hello, world!"}, {"text": "How are you?"}]
    content_str = "\n".join(json.dumps(item) for item in content)
    content_bytes = content_str.encode()

    mock_request = mocker.MagicMock()
    mock_request.headers = {}  # response.request headers have to be set otherwise it will confuse the framework and not parse the response into an object

    # Mock response 1: POST /files (get presigned URL)
    mock_presigned_response = Response(
        status_code=302,
        headers={
            "Location": "https://mock-presigned-url.com",
            "X-Together-File-Id": "file-30b2f515-c146-4780-80e6-d8a84f4caaaa",
        },
        request=mock_request,
    )

    # Mock response 2: PUT to presigned URL (upload file)
    mock_upload_response = Response(
        status_code=200,
        request=mock_request,
    )

    # Mock response 3: POST /files/{file_id}/preprocess (finalize)
    mock_finalize_response = Response(
        status_code=200,
        json={
            "id": "file-30b2f515-c146-4780-80e6-d8a84f4caaaa",
            "bytes": len(content_str),
            "created_at": 1234567890,
            "filename": "valid.jsonl",
            "FileType": "jsonl",
            "LineCount": 0,
            "purpose": "fine-tune",
            "object": "file",
            "Processed": True,
        },
        request=mock_request,
    )

    mock_send_requestor = mocker.MagicMock()
    mock_send_requestor.side_effect = [mock_presigned_response, mock_upload_response, mock_finalize_response]

    # Mock the post method directly on the client
    client = Together(api_key="fake_api_key")
    mocker.patch.object(client._client, "send", mock_send_requestor)
    files = client.files

    # Make a temporary file object
    file = tmp_path / "valid.jsonl"
    with file.open("w") as f:
        f.write(content_str)

    # Test run method
    response = files.upload(
        file,
        purpose="fine-tune",
    )

    # Verify the response
    assert isinstance(response, FileResponse)
    assert response.filename == "valid.jsonl"
    assert response.bytes == len(content_bytes)
    assert response.created_at == 1234567890
    assert response.file_type == "jsonl"
    assert response.line_count == 0
    assert response.object == "file"
    assert response.processed == True
    assert response.purpose == "fine-tune"



from __future__ import annotations

from typing import Literal

import click

from together import Together, omit
from together.lib.cli.api._utils import handle_api_errors
from together.lib.utils.serializer import datetime_serializer


@click.command()
@click.option("--json", is_flag=True, help="Print output in JSON format")
@click.option(
    "--type",
    type=click.Choice(["dedicated", "serverless"]),
    help="Filter by endpoint type",
)
@click.option(
    "--mine",
    type=click.BOOL,
    default=None,
    help="true (only mine), default=all",
)
@click.option(
    "--usage-type",
    type=click.Choice(["on-demand", "reserved"]),
    help="Filter by endpoint usage type",
)
@click.pass_context
@handle_api_errors("Endpoints")
def list(
    ctx: click.Context,
    json: bool,
    type: Literal["dedicated", "serverless"] | None,
    usage_type: Literal["on-demand", "reserved"] | None,
    mine: bool | None,
) -> None:
    """List all inference endpoints (includes both dedicated and serverless endpoints)."""
    client: Together = ctx.obj

    endpoints = client.endpoints.list(
        type=type or omit,
        usage_type=usage_type or omit,
        mine=mine if mine is not None else omit,
    )

    if not endpoints:
        click.echo("No dedicated endpoints found", err=True)
        return

    click.echo("Endpoints:", err=True)
    if json:
        import json as json_lib

        click.echo(
            json_lib.dumps(
                [endpoint.model_dump() for endpoint in endpoints.data], default=datetime_serializer, indent=2
            )
        )
    else:
        for endpoint in endpoints.data:
            ctx.obj.print_endpoint(
                endpoint,
            )
            click.echo()



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel

__all__ = ["EndpointListAvzonesResponse"]


class EndpointListAvzonesResponse(BaseModel):
    """List of unique availability zones"""

    avzones: List[str]



from typing import Any, Dict, List
from datetime import datetime, timezone
from textwrap import wrap

import click
from tabulate import tabulate

from together import Together
from together.lib.utils import finetune_price_to_dollars
from together.lib.cli.api._utils import handle_api_errors, generate_progress_bar


@click.command()
@click.pass_context
@handle_api_errors("Fine-tuning")
def list(ctx: click.Context) -> None:
    """List fine-tuning jobs"""
    client: Together = ctx.obj

    response = client.fine_tuning.list()

    response.data = response.data or []

    # Use a default datetime for None values to make sure the key function always returns a comparable value
    epoch_start = datetime.fromtimestamp(0, tz=timezone.utc)
    response.data.sort(key=lambda x: x.created_at or epoch_start)

    display_list: List[Dict[str, Any]] = []
    for i in response.data:
        display_list.append(
            {
                "Fine-tune ID": i.id,
                "Model Output Name": "\n".join(wrap(i.x_model_output_name or "", width=30)),
                "Status": i.status,
                "Created At": i.created_at,
                "Price": f"""${
                    finetune_price_to_dollars(float(str(i.total_price)))
                }""",  # convert to string for mypy typing
                "Progress": generate_progress_bar(i, datetime.now().astimezone(), use_rich=False),
            }
        )
    table = tabulate(display_list, headers="keys", tablefmt="grid", showindex=True)

    click.echo(table)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ChatCompletionStructuredMessageImageURLParam", "ImageURL"]


class ImageURL(TypedDict, total=False):
    url: Required[str]
    """The URL of the image"""


class ChatCompletionStructuredMessageImageURLParam(TypedDict, total=False):
    image_url: ImageURL

    type: Literal["image_url"]



import click

from .list import list
from .check import check
from .delete import delete
from .upload import upload
from .retrieve import retrieve
from .retrieve_content import retrieve_content


@click.group()
@click.pass_context
def files(ctx: click.Context) -> None:
    """File API commands"""
    pass


files.add_command(upload)
files.add_command(list)
files.add_command(retrieve)
files.add_command(retrieve_content)
files.add_command(delete)
files.add_command(check)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["Autoscaling"]


class Autoscaling(BaseModel):
    """Configuration for automatic scaling of replicas based on demand."""

    max_replicas: int
    """The maximum number of replicas to scale up to under load"""

    min_replicas: int
    """The minimum number of replicas to maintain, even when there is no load"""



import json

import click

from together import Together
from together.lib.cli.api._utils import handle_api_errors
from together.lib.utils.serializer import datetime_serializer


@click.command()
@click.pass_context
@click.argument("fine_tune_id", type=str, required=True)
@click.option("--quiet", is_flag=True, help="Do not prompt for confirmation before cancelling job")
@handle_api_errors("Fine-tuning")
def cancel(ctx: click.Context, fine_tune_id: str, quiet: bool = False) -> None:
    """Cancel fine-tuning job"""
    client: Together = ctx.obj
    if not quiet:
        confirm_response = input(
            "You will be billed for any completed training steps upon cancellation. "
            f"Do you want to cancel job {fine_tune_id}? [y/N]"
        )
        if "y" not in confirm_response.lower():
            click.echo({"status": "Cancel not submitted"})
            return
    response = client.fine_tuning.cancel(fine_tune_id)

    click.echo(json.dumps(response.model_dump(exclude_none=True), indent=4, default=datetime_serializer))



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ToolChoice", "Function"]


class Function(BaseModel):
    arguments: str

    name: str


class ToolChoice(BaseModel):
    id: str

    function: Function

    index: float

    type: Literal["function"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["FilePurpose"]

FilePurpose: TypeAlias = Literal[
    "fine-tune", "eval", "eval-sample", "eval-output", "eval-summary", "batch-generated", "batch-api"
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from typing_extensions import Literal, Required, TypedDict

from ..._types import FileTypes

__all__ = ["TranslationCreateParams"]


class TranslationCreateParams(TypedDict, total=False):
    file: Required[Union[FileTypes, str]]
    """Audio file upload or public HTTP/HTTPS URL.

    Supported formats .wav, .mp3, .m4a, .webm, .flac.
    """

    language: str
    """Target output language.

    Optional ISO 639-1 language code. If omitted, language is set to English.
    """

    model: Literal["openai/whisper-large-v3"]
    """Model to use for translation"""

    prompt: str
    """Optional text to bias decoding."""

    response_format: Literal["json", "verbose_json"]
    """The format of the response"""

    temperature: float
    """Sampling temperature between 0.0 and 1.0"""

    timestamp_granularities: Union[Literal["segment", "word"], List[Literal["segment", "word"]]]
    """Controls level of timestamp detail in verbose_json.

    Only used when response_format is verbose_json. Can be a single granularity or
    an array to get multiple levels.
    """



from typing import Any, Dict, List

import click
from tabulate import tabulate

from together import Together
from together.lib.utils.tools import format_timestamp
from together.lib.cli.api._utils import handle_api_errors


@click.command()
@click.pass_context
@click.argument("fine_tune_id", type=str, required=True)
@handle_api_errors("Fine-tuning")
def list_checkpoints(ctx: click.Context, fine_tune_id: str) -> None:
    """List available checkpoints for a fine-tuning job"""
    client: Together = ctx.obj

    checkpoints = client.fine_tuning.list_checkpoints(fine_tune_id)

    display_list: List[Dict[str, Any]] = []
    for checkpoint in checkpoints.data:
        name = (
            f"{fine_tune_id}:{checkpoint.step}"
            if "intermediate" in checkpoint.checkpoint_type.lower()
            else fine_tune_id
        )
        display_list.append(
            {
                "Type": checkpoint.checkpoint_type,
                "Timestamp": format_timestamp(checkpoint.created_at),
                "Name": name,
            }
        )

    if display_list:
        click.echo(f"Job {fine_tune_id} contains the following checkpoints:")
        table = tabulate(display_list, headers="keys", tablefmt="grid")
        click.echo(table)
        click.echo("\nTo download a checkpoint, use `together fine-tuning download`")
    else:
        click.echo(f"No checkpoints found for job {fine_tune_id}")



# Manually added to minimize breaking changes from V1
from ._constants import (
    MAX_RETRY_DELAY as MAX_RETRY_DELAY,
    DEFAULT_MAX_RETRIES,
    INITIAL_RETRY_DELAY as INITIAL_RETRY_DELAY,
)
from .lib.constants import (
    MIN_SAMPLES as MIN_SAMPLES,
    DISABLE_TQDM as DISABLE_TQDM,
    MAX_IMAGE_BYTES as MAX_IMAGE_BYTES,
    NUM_BYTES_IN_GB as NUM_BYTES_IN_GB,
    MAX_FILE_SIZE_GB as MAX_FILE_SIZE_GB,
    MIN_PART_SIZE_MB as MIN_PART_SIZE_MB,
    DOWNLOAD_BLOCK_SIZE as DOWNLOAD_BLOCK_SIZE,
    MAX_MULTIPART_PARTS as MAX_MULTIPART_PARTS,
    TARGET_PART_SIZE_MB as TARGET_PART_SIZE_MB,
    MAX_CONCURRENT_PARTS as MAX_CONCURRENT_PARTS,
    MAX_IMAGES_PER_EXAMPLE as MAX_IMAGES_PER_EXAMPLE,
    MULTIPART_THRESHOLD_GB as MULTIPART_THRESHOLD_GB,
    MAX_BASE64_IMAGE_LENGTH as MAX_BASE64_IMAGE_LENGTH,
    MULTIPART_UPLOAD_TIMEOUT as MULTIPART_UPLOAD_TIMEOUT,
    PARQUET_EXPECTED_COLUMNS as PARQUET_EXPECTED_COLUMNS,
    REQUIRED_COLUMNS_MESSAGE as REQUIRED_COLUMNS_MESSAGE,
    JSONL_REQUIRED_COLUMNS_MAP as JSONL_REQUIRED_COLUMNS_MAP,
    POSSIBLE_ROLES_CONVERSATION as POSSIBLE_ROLES_CONVERSATION,
    DatasetFormat as DatasetFormat,
)

TIMEOUT_SECS = 600
MAX_SESSION_LIFETIME_SECS = 180
MAX_CONNECTION_RETRIES = 2
MAX_RETRIES = DEFAULT_MAX_RETRIES

BASE_URL = "https://api.together.xyz/v1"



import click

from together import Together
from together.lib.cli.api._utils import handle_api_errors


@click.command()
@click.argument("endpoint-id", required=True)
@click.pass_obj
@handle_api_errors("Endpoints")
def delete(client: Together, endpoint_id: str) -> None:
    """Delete a dedicated inference endpoint."""
    client.endpoints.delete(endpoint_id)
    click.echo("Successfully deleted endpoint", err=True)
    click.echo(endpoint_id)



import click

from .list import list
from .create import create
from .status import status
from .retrieve import retrieve


@click.group()
@click.pass_context
def evals(ctx: click.Context) -> None:
    """Evals API commands"""
    pass


evals.add_command(create)
evals.add_command(list)
evals.add_command(retrieve)
evals.add_command(status)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .file_response import FileResponse

__all__ = ["FileList"]


class FileList(BaseModel):
    data: List[FileResponse]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import eval_list_params, eval_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.evaluation_job import EvaluationJob
from ..types.eval_list_response import EvalListResponse
from ..types.eval_create_response import EvalCreateResponse
from ..types.eval_status_response import EvalStatusResponse

__all__ = ["EvalsResource", "AsyncEvalsResource"]


class EvalsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EvalsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return EvalsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EvalsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return EvalsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        parameters: eval_create_params.Parameters,
        type: Literal["classify", "score", "compare"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvalCreateResponse:
        """
        Create an evaluation job

        Args:
          parameters: Type-specific parameters for the evaluation

          type: The type of evaluation to perform

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/evaluation",
            body=maybe_transform(
                {
                    "parameters": parameters,
                    "type": type,
                },
                eval_create_params.EvalCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvalCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvaluationJob:
        """
        Get evaluation job details

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/evaluation/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationJob,
        )

    def list(
        self,
        *,
        limit: int | Omit = omit,
        status: str | Omit = omit,
        user_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvalListResponse:
        """
        Get all evaluation jobs

        Args:
          user_id: Admin users can specify a user ID to filter jobs. Pass empty string to get all
              jobs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/evaluation",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "status": status,
                        "user_id": user_id,
                    },
                    eval_list_params.EvalListParams,
                ),
            ),
            cast_to=EvalListResponse,
        )

    def status(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvalStatusResponse:
        """
        Get evaluation job status and results

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/evaluation/{id}/status",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvalStatusResponse,
        )


class AsyncEvalsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEvalsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return AsyncEvalsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEvalsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return AsyncEvalsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        parameters: eval_create_params.Parameters,
        type: Literal["classify", "score", "compare"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvalCreateResponse:
        """
        Create an evaluation job

        Args:
          parameters: Type-specific parameters for the evaluation

          type: The type of evaluation to perform

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/evaluation",
            body=await async_maybe_transform(
                {
                    "parameters": parameters,
                    "type": type,
                },
                eval_create_params.EvalCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvalCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvaluationJob:
        """
        Get evaluation job details

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/evaluation/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationJob,
        )

    async def list(
        self,
        *,
        limit: int | Omit = omit,
        status: str | Omit = omit,
        user_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvalListResponse:
        """
        Get all evaluation jobs

        Args:
          user_id: Admin users can specify a user ID to filter jobs. Pass empty string to get all
              jobs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/evaluation",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "status": status,
                        "user_id": user_id,
                    },
                    eval_list_params.EvalListParams,
                ),
            ),
            cast_to=EvalListResponse,
        )

    async def status(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvalStatusResponse:
        """
        Get evaluation job status and results

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/evaluation/{id}/status",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvalStatusResponse,
        )


class EvalsResourceWithRawResponse:
    def __init__(self, evals: EvalsResource) -> None:
        self._evals = evals

        self.create = to_raw_response_wrapper(
            evals.create,
        )
        self.retrieve = to_raw_response_wrapper(
            evals.retrieve,
        )
        self.list = to_raw_response_wrapper(
            evals.list,
        )
        self.status = to_raw_response_wrapper(
            evals.status,
        )


class AsyncEvalsResourceWithRawResponse:
    def __init__(self, evals: AsyncEvalsResource) -> None:
        self._evals = evals

        self.create = async_to_raw_response_wrapper(
            evals.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            evals.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            evals.list,
        )
        self.status = async_to_raw_response_wrapper(
            evals.status,
        )


class EvalsResourceWithStreamingResponse:
    def __init__(self, evals: EvalsResource) -> None:
        self._evals = evals

        self.create = to_streamed_response_wrapper(
            evals.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            evals.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            evals.list,
        )
        self.status = to_streamed_response_wrapper(
            evals.status,
        )


class AsyncEvalsResourceWithStreamingResponse:
    def __init__(self, evals: AsyncEvalsResource) -> None:
        self._evals = evals

        self.create = async_to_streamed_response_wrapper(
            evals.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            evals.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            evals.list,
        )
        self.status = async_to_streamed_response_wrapper(
            evals.status,
        )



import click

from together import Together
from together.lib.cli.api._utils import handle_api_errors


@click.command()
@click.pass_context
@click.argument("id", type=str, required=True)
@click.option("--output", type=str, default=None, help="Output filename")
@handle_api_errors("Files")
def retrieve_content(ctx: click.Context, id: str, output: str) -> None:
    """Retrieve file content and output to file"""

    client: Together = ctx.obj

    response = client.files.content(id=id)

    if output:
        with open(output, "wb") as f:
            f.write(response.read())
        click.echo(f"File saved to {output}")

    else:
        click.echo(response.read().decode("utf-8"))



from __future__ import annotations

import json as json_lib
from typing import Literal

import click

from together import Together, omit
from together.lib.cli.api._utils import handle_api_errors


@click.command()
@click.argument("cluster-id", required=True)
@click.option(
    "--num-gpus",
    type=int,
    help="Number of GPUs to allocate in the cluster",
)
@click.option(
    "--cluster-type",
    type=click.Choice(["KUBERNETES", "SLURM"]),
    help="Cluster type",
)
@click.option(
    "--json",
    is_flag=True,
    help="Output in JSON format",
)
@click.pass_context
@handle_api_errors("Clusters")
def update(
    ctx: click.Context,
    cluster_id: str,
    num_gpus: int | None = None,
    cluster_type: Literal["KUBERNETES", "SLURM"] | None = None,
    json: bool = False,
) -> None:
    """Update a cluster"""
    client: Together = ctx.obj

    if not json:
        click.echo("Clusters: Updating cluster...")

    client.beta.clusters.update(
        cluster_id,
        num_gpus=num_gpus if num_gpus is not None else omit,
        cluster_type=cluster_type if cluster_type is not None else omit,
    )

    if json:
        cluster = client.beta.clusters.retrieve(cluster_id)
        click.echo(json_lib.dumps(cluster.model_dump(exclude_none=True), indent=4))
    else:
        click.echo("Clusters: Done")



from together.types.chat.completion_create_params import (
    MessageChatCompletionUserMessageParam,
    ChatCompletionStructuredMessageTextParam,
    ChatCompletionStructuredMessageVideoURLParam,
)
from together.types.chat.chat_completion_structured_message_video_url_param import VideoURL


def test_video_url_message():
    # Test creating a message with video_url content
    message = MessageChatCompletionUserMessageParam(
        role="user",
        content=[
            ChatCompletionStructuredMessageTextParam(type="text", text="What's in this video?"),
            ChatCompletionStructuredMessageVideoURLParam(
                type="video_url",
                video_url=VideoURL(url="https://example.com/video.mp4"),
            ),
        ],
    )

    # Verify the message structure
    assert message["role"] == "user"
    assert isinstance(message["content"], list)
    assert len(message["content"]) == 2

    # Verify text content
    assert "type" in message["content"][0]
    assert message["content"][0]["type"] == "text"
    assert message["content"][0]["text"] == "What's in this video?"

    # Verify video_url content
    assert "type" in message["content"][1]
    assert message["content"][1]["type"] == "video_url"
    assert message["content"][1]["video_url"]["url"] == "https://example.com/video.mp4"



from typing import Union, Literal

import pytest

from together.lib.types.fine_tuning import (
    FullTrainingType,
    LoRATrainingType,
    TrainingMethodSFT,
    FinetuneTrainingLimits,
    FinetuneFullTrainingLimits,
    FinetuneLoraTrainingLimits,
)
from together.lib.resources.fine_tuning import create_finetune_request

_MODEL_NAME = "meta-llama/Meta-Llama-3.1-8B-Instruct-Reference"
_TRAINING_FILE = "file-7dbce5e9-7993-4520-9f3e-a7ece6c39d84"
_VALIDATION_FILE = "file-7dbce5e9-7553-4520-9f3e-a7ece6c39d84"
_FROM_CHECKPOINT = "ft-12345678-1234-1234-1234-1234567890ab"
_MODEL_LIMITS = FinetuneTrainingLimits(
    max_num_epochs=20,
    max_learning_rate=1.0,
    min_learning_rate=1e-6,
    full_training=FinetuneFullTrainingLimits(
        max_batch_size=96,
        max_batch_size_dpo=48,
        min_batch_size=8,
    ),
    lora_training=FinetuneLoraTrainingLimits(
        max_batch_size=128,
        max_batch_size_dpo=64,
        min_batch_size=8,
        max_rank=64,
        target_modules=["q", "k", "v", "o", "mlp"],
    ),
)


def test_simple_request():
    request, _, _ = create_finetune_request(
        model_limits=_MODEL_LIMITS,
        model=_MODEL_NAME,
        training_file=_TRAINING_FILE,
    )

    assert request.model == _MODEL_NAME
    assert request.training_file == _TRAINING_FILE
    assert request.learning_rate > 0
    assert request.n_epochs > 0
    assert request.warmup_ratio == 0.0
    assert request.training_type is not None
    assert request.training_type.type == "Full"
    assert request.batch_size == "max"


def test_validation_file():
    request, _, _ = create_finetune_request(
        model_limits=_MODEL_LIMITS,
        model=_MODEL_NAME,
        training_file=_TRAINING_FILE,
        validation_file=_VALIDATION_FILE,
    )

    assert request.training_file == _TRAINING_FILE
    assert request.validation_file == _VALIDATION_FILE


def test_no_training_file():
    with pytest.raises(TypeError, match="missing 1 required positional argument: 'training_file'"):
        _ = create_finetune_request(  # type: ignore
            model_limits=_MODEL_LIMITS,
            model=_MODEL_NAME,
        )


def test_lora_request():
    request, _, _ = create_finetune_request(
        model_limits=_MODEL_LIMITS,
        model=_MODEL_NAME,
        training_file=_TRAINING_FILE,
        lora=True,
    )

    assert isinstance(request.training_type, LoRATrainingType)
    assert request.training_type.type == "Lora"
    assert _MODEL_LIMITS.lora_training is not None
    assert request.training_type.lora_r == _MODEL_LIMITS.lora_training.max_rank
    assert request.training_type.lora_alpha == _MODEL_LIMITS.lora_training.max_rank * 2
    assert request.training_type.lora_dropout == 0.0
    assert request.training_type.lora_trainable_modules == "all-linear"
    assert request.batch_size == "max"


@pytest.mark.parametrize("lora_dropout", [-1, 0, 0.5, 1.0, 10.0])
def test_lora_request_with_lora_dropout(lora_dropout: float):
    if 0 <= lora_dropout < 1:
        request, _, _ = create_finetune_request(
            model_limits=_MODEL_LIMITS,
            model=_MODEL_NAME,
            training_file=_TRAINING_FILE,
            lora=True,
            lora_dropout=lora_dropout,
        )
        assert isinstance(request.training_type, LoRATrainingType)
        assert request.training_type.lora_dropout == lora_dropout
    else:
        with pytest.raises(
            ValueError,
            match=r"LoRA dropout must be in \[0, 1\) range.",
        ):
            create_finetune_request(
                model_limits=_MODEL_LIMITS,
                model=_MODEL_NAME,
                training_file=_TRAINING_FILE,
                lora=True,
                lora_dropout=lora_dropout,
            )


def test_dpo_request_lora():
    request, _, _ = create_finetune_request(
        model_limits=_MODEL_LIMITS,
        model=_MODEL_NAME,
        training_file=_TRAINING_FILE,
        training_method="dpo",
        lora=True,
    )

    assert isinstance(request.training_type, LoRATrainingType)
    assert request.training_type.type == "Lora"
    assert _MODEL_LIMITS.lora_training is not None
    assert request.training_type.lora_r == _MODEL_LIMITS.lora_training.max_rank
    assert request.training_type.lora_alpha == _MODEL_LIMITS.lora_training.max_rank * 2
    assert request.training_type.lora_dropout == 0.0
    assert request.training_type.lora_trainable_modules == "all-linear"
    assert request.batch_size == "max"


def test_dpo_request():
    request, _, _ = create_finetune_request(
        model_limits=_MODEL_LIMITS,
        model=_MODEL_NAME,
        training_file=_TRAINING_FILE,
        training_method="dpo",
        lora=False,
    )

    assert isinstance(request.training_type, FullTrainingType)
    assert request.training_type.type == "Full"
    assert request.batch_size == "max"


def test_from_checkpoint_request():
    request, _, _ = create_finetune_request(
        model_limits=_MODEL_LIMITS,
        training_file=_TRAINING_FILE,
        from_checkpoint=_FROM_CHECKPOINT,
    )

    assert request.model is None
    assert request.from_checkpoint == _FROM_CHECKPOINT


def test_both_from_checkpoint_model_name():
    with pytest.raises(
        ValueError,
        match="You must specify either a model or a checkpoint to start a job from, not both",
    ):
        _ = create_finetune_request(
            model_limits=_MODEL_LIMITS,
            model=_MODEL_NAME,
            training_file=_TRAINING_FILE,
            from_checkpoint=_FROM_CHECKPOINT,
        )


def test_no_from_checkpoint_no_model_name():
    with pytest.raises(ValueError, match="You must specify either a model or a checkpoint"):
        _ = create_finetune_request(
            model_limits=_MODEL_LIMITS,
            training_file=_TRAINING_FILE,
        )


@pytest.mark.parametrize("batch_size", [256, 1])
@pytest.mark.parametrize("use_lora", [False, True])
def test_batch_size_limit(batch_size: int, use_lora: bool):
    model_limits = _MODEL_LIMITS.full_training if not use_lora else _MODEL_LIMITS.lora_training
    assert model_limits is not None
    max_batch_size = model_limits.max_batch_size
    min_batch_size = model_limits.min_batch_size

    if batch_size > max_batch_size:
        error_message = (
            f"Requested batch size of {batch_size} is higher that the maximum allowed value of {max_batch_size}"
        )
        with pytest.raises(ValueError, match=error_message):
            _ = create_finetune_request(
                model_limits=_MODEL_LIMITS,
                model=_MODEL_NAME,
                training_file=_TRAINING_FILE,
                batch_size=batch_size,
                lora=use_lora,
            )

    if batch_size < min_batch_size:
        error_message = (
            f"Requested batch size of {batch_size} is lower that the minimum allowed value of {min_batch_size}"
        )
        with pytest.raises(ValueError, match=error_message):
            _ = create_finetune_request(
                model_limits=_MODEL_LIMITS,
                model=_MODEL_NAME,
                training_file=_TRAINING_FILE,
                batch_size=batch_size,
                lora=use_lora,
            )


def test_non_lora_model():
    with pytest.raises(ValueError, match="LoRA adapters are not supported for the selected model."):
        _ = create_finetune_request(
            model_limits=FinetuneTrainingLimits(
                max_num_epochs=20,
                max_learning_rate=1.0,
                min_learning_rate=1e-6,
                full_training=FinetuneFullTrainingLimits(
                    max_batch_size=96,
                    max_batch_size_dpo=48,
                    min_batch_size=8,
                ),
                lora_training=None,
            ),
            model=_MODEL_NAME,
            training_file=_TRAINING_FILE,
            lora=True,
        )


def test_non_full_model():
    with pytest.raises(ValueError, match="Full training is not supported for the selected model."):
        _ = create_finetune_request(
            model_limits=FinetuneTrainingLimits(
                max_num_epochs=20,
                max_learning_rate=1.0,
                min_learning_rate=1e-6,
                lora_training=FinetuneLoraTrainingLimits(
                    max_batch_size=96,
                    max_batch_size_dpo=48,
                    min_batch_size=8,
                    max_rank=64,
                    target_modules=["q", "k", "v", "o", "mlp"],
                ),
                full_training=None,
            ),
            model=_MODEL_NAME,
            training_file=_TRAINING_FILE,
            lora=False,
        )


@pytest.mark.parametrize("warmup_ratio", [-1.0, 2.0])
def test_bad_warmup(warmup_ratio: float):
    with pytest.raises(ValueError, match="Warmup ratio should be between 0 and 1"):
        _ = create_finetune_request(
            model_limits=_MODEL_LIMITS,
            model=_MODEL_NAME,
            training_file=_TRAINING_FILE,
            warmup_ratio=warmup_ratio,
        )


@pytest.mark.parametrize("min_lr_ratio", [-1.0, 2.0])
def test_bad_min_lr_ratio(min_lr_ratio: float):
    with pytest.raises(ValueError, match="Min learning rate ratio should be between 0 and 1"):
        _ = create_finetune_request(
            model_limits=_MODEL_LIMITS,
            model=_MODEL_NAME,
            training_file=_TRAINING_FILE,
            min_lr_ratio=min_lr_ratio,
        )


@pytest.mark.parametrize("max_grad_norm", [-1.0, -0.01])
def test_bad_max_grad_norm(max_grad_norm: float):
    with pytest.raises(ValueError, match="Max gradient norm should be non-negative"):
        _ = create_finetune_request(
            model_limits=_MODEL_LIMITS,
            model=_MODEL_NAME,
            training_file=_TRAINING_FILE,
            max_grad_norm=max_grad_norm,
        )


@pytest.mark.parametrize("weight_decay", [-1.0, -0.01])
def test_bad_weight_decay(weight_decay: float):
    with pytest.raises(ValueError, match="Weight decay should be non-negative"):
        _ = create_finetune_request(
            model_limits=_MODEL_LIMITS,
            model=_MODEL_NAME,
            training_file=_TRAINING_FILE,
            weight_decay=weight_decay,
        )


def test_bad_training_method():
    with pytest.raises(ValueError, match="training_method must be one of .*"):
        _ = create_finetune_request(
            model_limits=_MODEL_LIMITS,
            model=_MODEL_NAME,
            training_file=_TRAINING_FILE,
            training_method="NON_SFT",
        )


@pytest.mark.parametrize("train_on_inputs", [True, False, "auto", None])
def test_train_on_inputs_for_sft(train_on_inputs: Union[bool, Literal["auto"], None]):
    request, _, _ = create_finetune_request(
        model_limits=_MODEL_LIMITS,
        model=_MODEL_NAME,
        training_file=_TRAINING_FILE,
        training_method="sft",
        train_on_inputs=train_on_inputs,
    )
    assert isinstance(request.training_method, TrainingMethodSFT)
    assert request.training_method.method == "sft"
    if isinstance(train_on_inputs, bool):
        assert request.training_method.train_on_inputs is train_on_inputs
    else:
        assert request.training_method.train_on_inputs == "auto"


def test_train_on_inputs_not_supported_for_dpo():
    with pytest.raises(ValueError, match="train_on_inputs is only supported for SFT training"):
        _ = create_finetune_request(
            model_limits=_MODEL_LIMITS,
            model=_MODEL_NAME,
            training_file=_TRAINING_FILE,
            training_method="dpo",
            train_on_inputs=True,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["JobListResponse", "Data", "DataArgs", "DataStatusUpdate"]


class DataArgs(BaseModel):
    description: Optional[str] = None

    x_model_name: Optional[str] = FieldInfo(alias="modelName", default=None)

    x_model_source: Optional[str] = FieldInfo(alias="modelSource", default=None)


class DataStatusUpdate(BaseModel):
    message: str

    status: str

    timestamp: datetime


class Data(BaseModel):
    args: DataArgs

    created_at: datetime

    job_id: str

    status: Literal["Queued", "Running", "Complete", "Failed"]

    status_updates: List[DataStatusUpdate]

    type: str

    updated_at: datetime


class JobListResponse(BaseModel):
    data: List[Data]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel

__all__ = [
    "TranscriptionCreateResponse",
    "AudioTranscriptionJsonResponse",
    "AudioTranscriptionVerboseJsonResponse",
    "AudioTranscriptionVerboseJsonResponseSegment",
    "AudioTranscriptionVerboseJsonResponseSpeakerSegment",
    "AudioTranscriptionVerboseJsonResponseSpeakerSegmentWord",
    "AudioTranscriptionVerboseJsonResponseWord",
]


class AudioTranscriptionJsonResponse(BaseModel):
    text: str
    """The transcribed text"""


class AudioTranscriptionVerboseJsonResponseSegment(BaseModel):
    id: int
    """Unique identifier for the segment"""

    end: float
    """End time of the segment in seconds"""

    start: float
    """Start time of the segment in seconds"""

    text: str
    """The text content of the segment"""


class AudioTranscriptionVerboseJsonResponseSpeakerSegmentWord(BaseModel):
    end: float
    """End time of the word in seconds"""

    start: float
    """Start time of the word in seconds"""

    word: str
    """The word"""

    speaker_id: Optional[str] = None
    """The speaker id for the word (only when diarize is enabled)"""


class AudioTranscriptionVerboseJsonResponseSpeakerSegment(BaseModel):
    id: int
    """Unique identifier for the speaker segment"""

    end: float
    """End time of the speaker segment in seconds"""

    speaker_id: str
    """The speaker identifier"""

    start: float
    """Start time of the speaker segment in seconds"""

    text: str
    """The full text spoken by this speaker in this segment"""

    words: List[AudioTranscriptionVerboseJsonResponseSpeakerSegmentWord]
    """Array of words spoken by this speaker in this segment"""


class AudioTranscriptionVerboseJsonResponseWord(BaseModel):
    end: float
    """End time of the word in seconds"""

    start: float
    """Start time of the word in seconds"""

    word: str
    """The word"""

    speaker_id: Optional[str] = None
    """The speaker id for the word (only when diarize is enabled)"""


class AudioTranscriptionVerboseJsonResponse(BaseModel):
    duration: float
    """The duration of the audio in seconds"""

    language: str
    """The language of the audio"""

    segments: List[AudioTranscriptionVerboseJsonResponseSegment]
    """Array of transcription segments"""

    task: Literal["transcribe", "translate"]
    """The task performed"""

    text: str
    """The transcribed text"""

    speaker_segments: Optional[List[AudioTranscriptionVerboseJsonResponseSpeakerSegment]] = None
    """Array of transcription speaker segments (only when diarize is enabled)"""

    words: Optional[List[AudioTranscriptionVerboseJsonResponseWord]] = None
    """
    Array of transcription words (only when timestamp_granularities includes 'word')
    """


TranscriptionCreateResponse: TypeAlias = Union[AudioTranscriptionJsonResponse, AudioTranscriptionVerboseJsonResponse]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel
from .cluster_storage import ClusterStorage

__all__ = ["StorageListResponse"]


class StorageListResponse(BaseModel):
    volumes: List[ClusterStorage]



# Manually added to minimize breaking changes from V1
from ._exceptions import (
    APIError as APIError,
    RateLimitError as RateLimitError,
    APITimeoutError,
    BadRequestError,
    APIConnectionError as APIConnectionError,
    AuthenticationError as AuthenticationError,
    APIResponseValidationError,
)

Timeout = APITimeoutError
InvalidRequestError = BadRequestError
TogetherException = APIError
ResponseError = APIResponseValidationError



import sys
from typing import Union

from together.lib.utils._log import log_info


def get_google_colab_secret(secret_name: str = "TOGETHER_API_KEY") -> Union[str, None]:
    """
    Checks to see if the user is running in Google Colab, and looks for the Together API Key secret.

    Args:
        secret_name (str, optional). Defaults to TOGETHER_API_KEY

    Returns:
        str: if the API key is found; None if an error occurred or the secret was not found.
    """
    # If running in Google Colab, check for Together in notebook secrets
    if "google.colab" in sys.modules:
        from google.colab import userdata  # type: ignore

        try:
            api_key = userdata.get(secret_name)  # type: ignore
            if not isinstance(api_key, str):
                return None
            else:
                return str(api_key)
        except userdata.NotebookAccessError:  # type: ignore
            log_info(
                "The TOGETHER_API_KEY Colab secret was found, but notebook access is disabled. Please enable notebook "
                "access for the secret."
            )
        except userdata.SecretNotFoundError:  # type: ignore
            # warn and carry on
            log_info("Colab: No Google Colab secret named TOGETHER_API_KEY was found.")

        return None

    else:
        return None



import base64

from together import Together
from together.types.image_data_b64 import ImageDataB64

client = Together(api_key="04cf1e314be9c686cd14b3881f5c4ad76505af4c93a8d3fe6ef62337114d1d51")

image = client.images.generate(
    model="runwayml/stable-diffusion-v1-5",
    prompt="space robots",
    n=1,
)

# Write the image to a file
if image.data and image.data[0] and isinstance(image.data[0], ImageDataB64):
    image_data = image.data[0].b64_json

    binary_data = base64.b64decode(image_data)

    with open("image.jpg", "wb") as f:
        f.write(binary_data)
